[PACKAGE]: ../../raw/master/mkp/nvdct-0.9.9-20250214.mkp "nvdct-0.9.9-20250214.mkp"
# Network Visualization Data Creation Tool (NVDCT)

This script creates the topology data file needed for the [Checkmk Exchange Network visualization](https://exchange.checkmk.com/p/network-visualization) plugin.\
For more information see [Checkmk forum Network Visualization](https://forum.checkmk.com/t/network-visualization/41680) or the blog article [Build a Network Layer Visualization with Checkmk](https://checkmk.com/blog/network-layer-visualization-with-checkmk)

### Features:
- Create Layer 2 (CDP/LLDP) topology data 
- Create Layer 3 (IPv4) topology data
- Highlight conetion issues (Speed, Duplex/Half duplex, Native VLAN)
- Read connection data from the checkmk HW/SW inventory.
- Works in standalone and distributed monitoring environments
- Add custom connections (STATIC) for connections that are not in the inventory
- Optimized for my [CDP](https://thl-cmk.hopto.org/gitlab/checkmk/vendor-independent/inventory/inv_cdp_cache/), [LLDP](https://thl-cmk.hopto.org/gitlab/checkmk/vendor-independent/inventory/inv_lldp_cache/) and [IP](https://thl-cmk.hopto.org/gitlab/checkmk/vendor-independent/inventory/inv_ip_addresses) inventory plugins
- Can also be used with custom inventory plugins (deprecated)

The necessary HW/SW inventory data can be created with my CDP/LLDP/IPv4 inventory plugins.
- [CDP inventory plugin](https://thl-cmk.hopto.org/gitlab/checkmk/vendor-independent/inventory/inv_cdp_cache/)
- [LLDP inventory plugin](https://thl-cmk.hopto.org/gitlab/checkmk/vendor-independent/inventory/inv_lldp_cache/)
- [IP inventory plugin (SNMP)](https://thl-cmk.hopto.org/gitlab/checkmk/vendor-independent/inventory/inv_ip_addresses)
- [IP inventory plugin (Windows)](https://thl-cmk.hopto.org/gitlab/checkmk/vendor-independent/inventory/inv_win_if_ip)
- [IP inventory plugin (Linux)](https://thl-cmk.hopto.org/gitlab/checkmk/vendor-independent/inventory/inv_lnx_if_ip)
- [Interface name inventory plugin (optional but recommended)](https://thl-cmk.hopto.org/gitlab/checkmk/vendor-independent/inventory/inv_ifname)

---
### Acknowledgements

Andreas Boesl and schnetz for the Network visualization plugin\
detlev[dot]gerke[at]siegfried[dot]ch, who was my beta tester ;-)

---
### HowTo and Troubleshooting

For first steps on howto use the Network Visualization Data Creation Tool (NVDCT) set the [HowTo](HOWTO.md). There is also a little [Troubleshooting Guide](TROUBLESHOOTING.md) available.

---
### Download

* [Download latest mkp file][PACKAGE]

**Note**: before updating to a newer version check the [CHANGELOG](/CHANGELOG "CHANGELOG"). There could be always incompatible changes.

**Note**: NVDCT version 0.9.0 has dropped support for the CMK 2.2.0 Network Visualization data format.

---
### Installation

You can install the package by uploading it to your CheckMK site and as site user run 
```
mkp add PAKAGE_NAME.mkp
mkp enable PAKAGE_NAME VERSION
```
In the non RAW editions of CheckMK you can use the GUI to install the package (_Setup_ -> _Extension Packages_ -> _Upload package_)

---
### Want to Contribute?

Nice ;-) Have a look at the [contribution guidelines](CONTRIBUTING.md "Contributing")

---
### Requirements

To use this tool:
- your Checkmk Site must be running at least CMK version 2.3.0p1
- the HW/SW inventory must be enabled
- you need to install/enable an inventory plugin to collect the connection data I.e. my [CDP cache](https://thl-cmk.hopto.org/gitlab/checkmk/vendor-independent/inventory/inv_cdp_cache/) or [LLDP cache](https://thl-cmk.hopto.org/gitlab/checkmk/vendor-independent/inventory/inv_lldp_cache/) plugins
- check if the Checkmk HW/SW inventory finds the required data (CDP/LLDP neighbors)

**Note**: if you are using my CDP/LLDP/IPv4 plugins update at least to version
- 0.7.6-20240414 for CDP
- 0.9.7-20240414 for LLDP
- 0.0.4-20241210 for IPv4 (Linux)
- 0.0.6-20241210 for IPv4 (SNMP)
- 0.0.3-20241210 for IPv4 (Windows)

and let Checkmk run the HW/SW inventory for each device at least once

---
### Usage


Usage with LLDP:

```
~/local/bin/nvdct/nvdct.py -d -l LLDP
```

<details><summary>Help output from nvdct.py --help</summary>

```
OMD[build]:~$ ~/local/bin/nvdct/nvdct.py --help
usage: nvdct.py [-h] [-b {LIVESTATUS,MULTISITE,RESTAPI}] [-c CONFIG] [-d]
                [-l {CDP,LLDP,L3v4,STATIC} [{CDP,LLDP,L3v4,STATIC} ...]] [-o OUTPUT_DIRECTORY] [-v] [--api-port API_PORT]
                [--check-config] [--dont-compare] [--filter-customers {INCLUDE,EXCLUDE}] [--filter-sites {EXCLUDE,EXCLUDE}]
                [--keep-max-topologies KEEP_MAX_TOPOLOGIES] [--l2-case {INSENSITIVE,LOWER,UPPER,AUTO,OFF}] [--l2-display-ports]
                [--l2-display-neighbours] [--l2-prefix L2_PREFIX] [--l2-remove-domain {OFF,ON,AUTO}] [--l2-skip-external]
                [--l3-display-devices] [--l3-include-hosts] [--l3-include-loopback] [--l3-skip-cidr-0] [--l3-skip-cidr-32-128]
                [--l3-skip-if] [--l3-skip-ip] [--l3-skip-public] [--log-file LOG_FILE]
                [--log-level {CRITICAL,FATAL,ERROR,WARNING,INFO,DEBUG,OFF}] [--log-to-stdout]
                [--min-topology-age MIN_TOPOLOGY_AGE] [--pre-fetch] [--quiet] [--time-format TIME_FORMAT] [--update-config]

This script creates the topology data file needed for the Checkmk Network Visualization,
For more information see the announcement from schnetz in the Checkmk forum:
https://forum.checkmk.com/t/network-visualization/41680

The required plugins to create the inventory data can be found here:
https://thl-cmk.hopto.org/gitlab/explore/projects/topics/Network%20Visualization

Version: 0.9.8-20250205 | Written by: thl-cmk
for more information see: https://thl-cmk.hopto.org/gitlab/checkmk/vendor-independent/nvdct

options:
  -h, --help            show this help message and exit
  -b {LIVESTATUS,MULTISITE,RESTAPI}, --backend {LIVESTATUS,MULTISITE,RESTAPI}
                        Backend used to retrieve the topology data
                         - LIVESTATUS : fetches data via local Livestatus (local site only)
                         - MULTISITE  : like LIVESTATUS but for distributed environments (default)
                         - RESTAPI    : uses the Checkmk REST API.
  -c CONFIG, --config CONFIG
                        Set the name of the config file.
                        Default is ~/local/bin/nvdct/conf/nvdct.toml
  -d, --default         Set the created topology data as default. Will be created automatically
                        if it doesnt exists.
  -l {CDP,LLDP,L3v4,STATIC} [{CDP,LLDP,L3v4,STATIC} ...], --layers {CDP,LLDP,L3v4,STATIC} [{CDP,LLDP,L3v4,STATIC} ...]
                         - CDP    : needs inv_cdp_cache package at least in version 0.7.1-20240320
                         - LLDP   : needs inv_lldp_cache package at least in version 0.9.3-20240320
                         - L3v4   : needs inv_ip_address package at least in version 0.0.6-20241210 for SNMP based hosts
                                    for Linux based hosts inv_lnx_ip_if in version 0.0.4-20241210
                                    for Windows based hosts inv_win_ip_if in version 0.0.3-20241210
                         - STATIC : creates a topology base on the "[STATIC_CONNECTIONS]" section in the TOML file
  -o OUTPUT_DIRECTORY, --output-directory OUTPUT_DIRECTORY
                        Directory name where to save the topology data.
                        I.e.: my_topology. Default is the actual date/time
                        in "--time-format" format.
                        NOTE: the directory is a sub directory under "~/var/check_mk/topology/data/"
  -v, --version         Print version of this script and exit
  --api-port API_PORT   TCP Port to access the REST API. By NVDCT will try to automatically
                        detect the site apache port.
  --check-config        Only tries to read/parse the config file from nvdct.toml and exits.
  --dont-compare        Do not compare the actual topology data with the default topology
                        data. By default, the actual topology is compared with the default
                        topology. If the data matches, the actual topology is not saved.
                        So, if you run this tool in a cron job, a new topology will be
                        created only if there was a change, unless you use "--dont-compare".
  --filter-customers {INCLUDE,EXCLUDE}
                        INCLUDE/EXCLUDE customer list "[FILTER_BY_CUSTOMER]" from TOML file.NOTE: MULTISITE backend only.
  --filter-sites {EXCLUDE,EXCLUDE}
                        INCLUDE/EXCLUDE site list "[FILTER_BY_SITE]" from TOML file.
  --keep-max-topologies KEEP_MAX_TOPOLOGIES
                        Number of topologies to keep. The oldest topologies above keep
                        will be deleted.
                        NOTE: The default/protected topologies will be kept always.
  --l2-case {INSENSITIVE,LOWER,UPPER,AUTO,OFF}
                        Change L2 neighbour name case before matching to Checkmk host
                        - OFF         : Do not change the case of the neighbour name
                        - INSENSITIVE : search for a matching host by ignoring the case of neighbour name and host name
                        - LOWER       : change to all lower case
                        - UPPER       : change to all upper case, without the domain part of the neighbour name
                        - AUTO        : try all the above variants
                        Default is let the case of the neighbour name untouched ("OFF").
                        Takes place after "--l2-remove-domain" and before "--l2-prefix"
  --l2-display-ports    Use L2 port names as display name for interfaces in L2 topologies
  --l2-display-neighbours
                        Use L2 neighbour name as display name in L2 topologies
  --l2-prefix L2_PREFIX
                        Prepends each L2 neighbour name with the prefix before matching to a Checkmk host name.
                        Takes place after "--l2-remove-domain" and "--l2-case"
  --l2-remove-domain {OFF,ON,AUTO}
                        Handle the the domain name part of a neighbour name before matching it to a Checkmk host.
                        - OFF : dont touch the neighbour name, keep host name and domain part
                        - ON  : will remove the domain part from the neighbour name, keep only the host name part
                        - AUTO : try all of the above variants
                        Default: "OFF". Takes place after "--l2-remove-domain" and before "--l2-prefix"
  --l2-skip-external    Skip L2 neighbours external to Checkmk (that have no matching host)
  --l3-display-devices  Use L3 device names as display name for interfaces in L3 topologies
  --l3-include-hosts    Include hosts (single IP objects) in layer 3 topologies
  --l3-include-loopback
                        Include loopback ip-addresses in layer 3 topologies
  --l3-skip-cidr-0      Skip ip-address with CIDR "/0" in layer 3 topologies
  --l3-skip-cidr-32-128
                        Skip ip-address with CIDR "/32" or "/128" in layer 3 topologies
  --l3-skip-if          Dont show interface in layer 3 topologies
  --l3-skip-ip          Dont show ip-addresses in layer 3 topologies
  --l3-skip-public      Skip public ip-addresses in layer 3 topologies
  --log-file LOG_FILE   Set the log file. Default is "~/var/log/nvdct.log"
  --log-level {CRITICAL,FATAL,ERROR,WARNING,INFO,DEBUG,OFF}
                        Sets the log level. The default is "WARNING"
  --log-to-stdout       Send log to stdout.
  --min-topology-age MIN_TOPOLOGY_AGE
                        The minimum number of days before a topology is deleted by "--keep-max-topologies"
  --pre-fetch           Try to fetch host data, with less API calls. Can improve RESTAPI backend
                        performance
  --quiet               Suppress all output to stdtout
  --time-format TIME_FORMAT
                        Format string to render the time. (default: "%Y-%m-%dT%H:%M:%S.%m")
  --update-config       Adjusts options in config file.

Exit codes:
 0 - No error
 1 - Bad options list
 2 - Bad TOML file format
 3 - Backend not implemented
 4 - Automation secret not found
 5 - No layer to work on

Usage:
~/local/bin/nvdct/nvdct.py -c ~/local/bin/nvdct/conf/my_nvdct.toml
OMD[build]:~$
```

</details>

---
### Known limitations


---
### FAQ

<details><summary>If i open the Network visualization, there is no default topology</summary>

\
To make the topology created by this tool the default use the option `-d` / `--default`

```
$ ~/local/bin/nvdct/nvdct.py -c ~/local/bin/conf/my_nvdct.toml -d
```

</details>

<details><summary>Can i automate the creation of the topology data</summary>

\
Yes. Add a cron job for the site user in `~/etc/cron.d`. I.e. to create a topology every hour create the file `nvdct` in `~/etc/cron.d`

```
0 * * * * $OMD_ROOT/local/bin/nvdct/nvdct.py -c $OMD_ROOT/local/bin/nvdct/conf/my_nvdct.toml -d --keep-max-topologies 10 --min-topology-age 14
```

This will run the tool every hour and create a new topology if there was a changee. It keeps 10 topologies and deletes old topologies only if they are older than 14 days.

Don't forget to restart the crontab service after creating the cron job.

```
OMD[build]:~/etc/cron.d$ omd restart crontab 
Temporary filesystem already mounted
Removing Crontab...OK
Initializing Crontab...OK
OMD[build]:~/etc/cron.d$ 
```

With crontab -l you get the list of active cron jobs

```
OMD[build]:~$ crontab -l | grep -A 3 -B 1  nvdct 
# ------------------------------------------------------------
# /omd/sites/build/etc/cron.d/nvdct
0 * * * * $OMD_ROOT/local/bin/nvdct/nvdct.py -c $OMD_ROOT/local/bin/nvdct/conf/my_nvdct.toml -d --keep-max-topologies 10 --min-topology-age 14

# ------------------------------------------------------------
```

**Note**: By default the tool will only create a new topology if the new topology differs from the default topology. The name for the new topology is by default the creation date and time.

</details>

<details><summary>Can I limit the number of topologies saved</summary>

\
Yes. If you use the option --keep <number of topologies to keep> the tool will delete all topologies over this number. I.e.

```
$ ~/local/bin/nvdct/nvdct.py -c ~/local/bin/conf/my_nvdct.toml --keep-max-topologies 10
```
will delete the oldest topologies until there only 10 topologies left.

**Note**: the default topology will never be deleted.
</details>

<details><summary>I will not delete topologies from the last X days</summary>

\
No problem. Just use the option `--min-topology-age` togeher with `--keep-max-topologies`. I.e. 
```
$ ~/local/bin/nvdct/nvdct.py -c ~/local/bin/conf/my_nvdct.toml --keep-max-topologies 10 --min-topology-age 30
``` 
will only delete a topology if it is older than 30 days.
</details>

<details><summary>Can the topology data creation be customized</summary>

\
Yes you can and you sould. Create a copy of the default `~/local/bin/nvdct/conf/nvdct.toml` file. Or provide your own config file (see Option -c, --config).

**Note**: This file uses [Tom's Obvious Minimal Language](https://toml.io/en/)

```
~/local/bin/nvdct/nvdct.py -c ~/local/bin/nvdct/conf/my_nvdct.toml
```

</details>

<details><summary>My Neighbor names from the inventory and checkmk host names do not match</summary>

\
If the neighbor names from the inventory and the checkmk host names do not match, the topology does not work. To fix this, add a mapping between the neighbor name and the checkmk host name to the **HOST_MAP** in the config file (by default `nvdct.toml`). I.e.

```
# map inventory neighbour name to Checkmk host name
[L2_NEIGHBOUR_TO_HOST_MAP]
inventory_neighbour1 = "cmk_host1"
inventory_neighbour2 = "cmk_host2"
inventory_neighbour3 = "cmk_host3"
```

</details>

<details><summary>In my inventory are invalid neighbor names</summary>

\
If you have invalid neighbor names in the inventory (i.e. "not advertised"), you can drop them by adding them to the **L2_DROP_NEIGHBOURS** list in the configuration file (by default `nvdct.toml`). I.e..

```
# drop neighbours with invalid names
L2_DROP_NEIGHBOURS = [
    "not advertised",
    "a nother invalid name",
]
```

If you need a more advanced option to remove neighbours from the topology, you can also use the `L2_NEIGHBOUR_REPLACE_REGEX` section.
Ie. to remove all neighbours advertised as MAC-address only use:

```
[L2_NEIGHBOUR_REPLACE_REGEX]
"^(([0-9a-fA-F]){2}[:.-]?){5}([0-9a-fA-F]){2}$" = ""
```

</details>


<details><summary>Not all connections are included in the inventory</summary>

\
If your inventory data does not contain all the required topology information, you can add static connections in the data file (by default `nvdct.toml`). This may be the case if not all of your devices use CDP/LLDP.

```
# user defined static connections
# [0-9-a-zA-Z\.\_\-]{1,253} -> host
STATIC_CONNECTIONS = [
    # valid entry formats
    # ["left_host", "left_service", "right_service", "right_host"],
    # connection: "left_host"<->"left_service"<->"right_service"<->"right_host"
    # ["left_host", "", "right_service", "right_host"],
    # connection: "left_host"<->"right_service"<->"right_host"
    # ["left_host", "left_service", "", "right_host"],
    # connection: "left_host"<->"left_service"<->"right_host"
    # ["left_host", "", "", "right_host"],
    # connection: "left_host"<->"right_host"
]
```

</details>

<details><summary>Can I use the tool with other inventory data plugins</summary>

\
No.

</details>

<details><summary>How are port names mapped to interface service names?</summary>

\
When mapping Layer 2 port names, the first step is to check whether the port name is in the host's list of interface items; if it is, the port name is used; if it is not, the port name is searched for in the inventory interface table. If the interface table is found it tries to find the correct item by comparing the list of items with the fields index, description, alias, physical address (MAC) and (if available) [name](https://thl-cmk.hopto.org/gitlab/checkmk/vendor-independent/inventory/inv_ifname). If we still can't find the item, we pad the index and try the search again. 
So most of the time we should find the correct interface service for a port.

</details>

<details><summary>Troubleshooting</summary>
First check your HW/SW inventory if it not contains the data needed to create the network topology data you need first to troubleshoot the inventory. If the inventory is ok you can use the buildin logging to troubleshoot the data creation. By default the tool will create the log file `~/var/log/nvdct.log`. The default log level set is to **WARNING**, so most information for everything thats not working should go to the log. If you want to change the logging see the options:

  * --log-file LOG_FILE   Set the log file. Default is ~/var/log/nvdct.log
  * --log-level {CRITICAL,ERROR,WARN,INFO,DEBUG}
                        Sets the loglevel. The default is "WARN"
  * --log-to-stdout       Send log to stdout.

For more troubleshooting see [TROUBLESHOOTING.md](/TROUBLESHOOTING.md)

</details>

---
### Sample Output

```
OMD[build]:~$ ~/local/bin/nvdct/nvdct.py -c ~/local/bin/nvdct/conf/my_nvdct.toml -d

Network Visualisation Data Creation Tool (NVDCT)
by thl-cmk[at]outlook[dot]com, version 0.9.2-20241117
see https://thl-cmk.hopto.org/gitlab/checkmk/vendor-independent/nvdct

Start time....: 2024-11-17T17:20:54.11
Source lldp...: Devices/Objects/Connections added 8/18/15
Source cdp....: Devices/Objects/Connections added 28/144/174
Source l3v4...: Devices/Objects/Connections added 2/141/140
Time taken....: 13.360647338/s
End time......: 2024-11-17T17:21:08.11

OMD[build]:~$ 
```

Sample network topology CDP/LLDP
![CDP/LLDP](img/sample-cmk2-3.png?raw=true "Sample network topology CDP/LLDP")

Sample network topology IPv4
![CDP/LLDP](img/sample-cmk2-3-ipv4.png?raw=true "Sample network topology IPv4")