#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2024-12-11
# File  : nvdct/lib/constants.py

from enum import Enum, unique, auto
from logging import Logger, getLogger
from os import environ
from typing import Final

#
NVDCT_VERSION: Final[str] = '0.9.9-20250207'
#
OMD_ROOT: Final[str] = environ["OMD_ROOT"]
#
API_PORT_DEFAULT: Final[int] = 5001
CACHE_INTERFACES_DATA: Final[str] = 'interface_data'
CMK_SITE_CONF: Final[str] = f'{OMD_ROOT}/etc/omd/site.conf'
CONFIG_FILE: Final[str] = 'nvdct.toml'
CONFIG_PATH_DEFAULT: Final[str] = f'{OMD_ROOT}/local/bin/nvdct/conf'
DATAPATH: Final[str] = f'{OMD_ROOT}/var/check_mk/topology/data'
LOGGER: Logger = getLogger('root)')
LOG_PATH_DEFAULT: Final[str] =  f'{OMD_ROOT}/var/log'
LOG_FILE_DEFAULT: Final[str] = f'{LOG_PATH_DEFAULT}/nvdct.log'
MIN_CMK_VERSION_POST: Final[str] = '2.3.0p23'
SCRIPT: Final[str] = '~/local/bin/nvdct/nvdct.py'
TIME_FORMAT_DEFAULT: Final[str] = '%Y-%m-%dT%H:%M:%S.%m'

TIME_FORMAT_ARGPARSER: Final[str] = TIME_FORMAT_DEFAULT.replace('%', '%%')


class EnumValue(Enum):
    def __get__(self, instance, owner):
        return self.value

    @classmethod
    def list(cls):
        return list(map(lambda c: c.value, cls))

@unique
class ExitCodes(EnumValue):
    OK = 0
    BAD_OPTION_LIST = auto()
    BAD_TOML_FORMAT = auto()
    BACKEND_NOT_IMPLEMENTED = auto()
    AUTOMATION_SECRET_NOT_FOUND = auto()
    NO_LAYER_CONFIGURED = auto()
    FILE_NOT_FOUND = auto()


@unique
class IPVersion(EnumValue):
    IPv4 = 4
    IPv6 = 6


@unique
class URLs(EnumValue):
    NVDCT: Final[str] = 'https://thl-cmk.hopto.org/gitlab/checkmk/vendor-independent/nvdct'
    # CDP: Final[str] = 'https://thl-cmk.hopto.org/gitlab/checkmk/vendor-independent/inventory/inv_cdp_cache'
    # LLDP: Final[str] = 'LLDP: https://thl-cmk.hopto.org/gitlab/checkmk/vendor-independent/inventory/inv_lldp_cach'
    # SNMP_IP_ADDRESS: Final[str] = 'https://thl-cmk.hopto.org/gitlab/checkmk/vendor-independent/inventory/inv_ip_address'
    # LINUX_SNM_APPRESS: Final[str] = 'https://thl-cmk.hopto.org/gitlab/checkmk/vendor-independent/inventory/inv_lnx_if_ip'
    # WINDOWS_IP_ADDRESS: Final[str] = 'https://thl-cmk.hopto.org/gitlab/checkmk/vendor-independent/inventory/inv_win_if_ip'
    TOPIC_NV: Final[str] = 'https://thl-cmk.hopto.org/gitlab/explore/projects/topics/Network%20Visualization'
    FORUM_SCHNETZ: Final[str] = 'https://forum.checkmk.com/t/network-visualization/41680'


@unique
class Backends(EnumValue):
    LIVESTATUS: Final[str] = 'LIVESTATUS'
    MULTISITE: Final[str] = 'MULTISITE'
    RESTAPI: Final[str] = 'RESTAPI'


@unique
class L2Case(EnumValue):
    AUTO: Final[str] = 'AUTO'
    INSENSITIVE: Final[str] = 'INSENSITIVE'
    LOWER: Final[str] = 'LOWER'
    UPPER: Final[str] = 'UPPER'
    OFF: Final[str] = 'OFF'


@unique
class L2RemoveDomain(EnumValue):
    ON: Final[str] = 'ON'
    OFF: Final[str] = 'OFF'
    AUTO: Final[str] = 'AUTO'

@unique
class L2IgnoreMismatch(EnumValue):
    DUPLEX: Final[str] = 'DUPLEX'
    SPEED: Final[str] = 'SPEED'
    VLAN: Final[str] = 'VLAN'

@unique
class CacheItems(EnumValue):
    inventory = 'inventory'
    interfaces = 'interfaces'


@unique
class CliLong(EnumValue):
    API_PORT: Final[str] = '--api-port'
    BACKEND: Final[str] = '--backend'
    CHECK_CONFIG: Final[str] = '--check-config'
    CONFIG: Final[str] = '--config'
    DEFAULT: Final[str] = '--default'
    DONT_COMPARE: Final[str] = '--dont-compare'
    FILTER_CUSTOMERS: Final[str] = '--filter-customers'
    FILTER_SITES: Final[str] = '--filter-sites'
    KEEP_MAX_TOPOLOGIES: Final[str] = '--keep-max-topologies'
    L2_CASE: Final[str] = '--l2-case'
    L2_DISPLAY_NEIGHBOURS: Final[str] = '--l2-display-neighbours'
    L2_DISPLAY_PORTS: Final[str] = '--l2-display-ports'
    L2_IGNORE_MISMATCH: Final[str] = '--l2-ignore-mismatch'
    L2_PREFIX: Final[str] = '--l2-prefix'
    L2_REMOVE_DOMAIN: Final[str] = '--l2-remove-domain'
    L2_SKIP_EXTERNAL: Final[str] = '--l2-skip-external'
    L3_DISPLAY_DEVICES: Final[str] = '--l3-display-devices'
    L3_INCLUDE_HOSTS: Final[str] = '--l3-include-hosts'
    L3_INCLUDE_LOOPBACK: Final[str] = '--l3-include-loopback'
    L3_SKIP_CIDR_0: Final[str] = '--l3-skip-cidr-0'
    L3_SKIP_CIDR_32_128: Final[str] = '--l3-skip-cidr-32-128'
    L3_SKIP_IF: Final[str] = '--l3-skip-if'
    L3_SKIP_IP: Final[str] = '--l3-skip-ip'
    L3_SKIP_PUBLIC: Final[str] = '--l3-skip-public'
    LAYERS: Final[str] = '--layers'
    LOG_FILE: Final[str] = '--log-file'
    LOG_LEVEL: Final[str] = '--log-level'
    LOG_TO_STDOUT: Final[str] = '--log-to-stdout'
    MIN_TOPOLOGY_AGE: Final[str] = '--min-topology-age'
    OUTPUT_DIRECTORY: Final[str] = '--output-directory'
    PRE_FETCH: Final[str] = '--pre-fetch'
    QUIET: Final[str] = '--quiet'
    TIME_FORMAT: Final[str] = '--time-format'
    UPDATE_CONFIG: Final[str] = '--update-config'
    VERSION: Final[str] = '--version'


@unique
class CliShort(EnumValue):
    BACKEND: Final[str] = '-b'
    CONFIG: Final[str] = '-c'
    DEFAULT: Final[str] = '-d'
    LAYERS: Final[str] = '-l'
    OUTPUT_DIRECTORY: Final[str] = '-o'
    VERSION: Final[str] = '-v'


@unique
class EmblemNames(EnumValue):
    HOST_NODE: Final[str] = 'host_node'
    IP_ADDRESS: Final[str] = 'ip_address'
    IP_NETWORK: Final[str] = 'ip_network'
    L3_REPLACE: Final[str] = 'l3_replace'
    L3_SUMMARIZE: Final[str] = 'l3_summarize'
    SERVICE_NODE: Final[str] = 'service_node'


@unique
class EmblemValues(EnumValue):
    ICON_AGGREGATION: Final[str] = 'icon_aggr'
    ICON_ALERT_UNREACHABLE: Final[str] = 'icon_alert_unreach'
    ICON_PLUGINS_CLOUD: Final[str] = 'icon_plugins_cloud'
    IP_ADDRESS_80: Final[str] = 'ip-address_80'
    IP_NETWORK_80: Final[str] = 'ip-network_80'


@unique
class HostLabels(EnumValue):
    CDP: Final[str] = "'nvdct/has_cdp_neighbours' 'yes'"
    L3V4_HOSTS: Final[str] = "'nvdct/l3v4_topology' 'host'"
    L3V4_ROUTER: Final[str] = "'nvdct/l3v4_topology' 'router'"
    L3V6_HOSTS: Final[str] = "'nvdct/l3v6_topology' 'host'"
    L3V6_ROUTER: Final[str] = "'nvdct/l3v6_topology' 'router'"
    LLDP: Final[str] = "'nvdct/has_lldp_neighbours' 'yes'"


@unique
class HostFilter(EnumValue):
    FOLDER: Final[str] = 'filename'
    LABELS: Final[str] = 'labels'
    TAGS: Final[str] = 'tags'


@unique
class LiveStatusOperator(EnumValue):
    EQUAL: Final[str] = '='
    SUPERSET: Final[str] = '~'


@unique
class IncludeExclude(EnumValue):
    INCLUDE: Final[str] = 'INCLUDE'
    EXCLUDE: Final[str] = 'EXCLUDE'


@unique
class L2InvColumns(EnumValue):
    NEIGHBOUR: Final[str] = 'neighbour_name'
    LOCALPORT: Final[str] = 'local_port'
    NEIGHBOURPORT: Final[str] = 'neighbour_port'
    NEIGHBOURID: Final[str] = 'neighbour_id'
    GLOBALID: Final[str] = 'local_id'
    GLOBALNAME: Final[str] = 'local_name'


@unique
class L3InvColumns(EnumValue):
    ADDRESS: Final[str] = 'address'
    DEVICE: Final[str] = 'device'
    CIDR: Final[str] = 'cidr'


@unique
class InvPaths(EnumValue):
    CDP: Final[str] = 'networking,cdp_cache,neighbours,Table,Rows'
    CDP_GLOBAL: Final[str] = 'networking,cdp_cache,Attributes,Pairs'
    INTERFACES: Final[str] = 'networking,interfaces,Table,Rows'
    L3: Final[str] = 'networking,addresses,Table,Rows'
    LLDP: Final[str] = 'networking,lldp_cache,neighbours,Table,Rows'
    LLDP_GLOBAL: Final[str] = 'networking,lldp_cache,Attributes,Pairs'


@unique
class Layers(EnumValue):
    CDP: Final[str] = 'CDP'
    LLDP: Final[str] = 'LLDP'
    L3V4: Final[str] = 'L3v4'
    L3V6: Final[str] = 'L3v6'
    STATIC: Final[str] = 'STATIC'


@unique
class LogLevels(EnumValue):
    CRITICAL: Final[str] = 'CRITICAL'
    FATAL: Final[str] = 'FATAL'
    ERROR: Final[str] = 'ERROR'
    WARNING: Final[str] = 'WARNING'
    INFO: Final[str] = 'INFO'
    DEBUG: Final[str] = 'DEBUG'
    OFF: Final[str] = 'OFF'


class MinVersions(EnumValue):
    CDP: Final[str] = '0.7.1-20240320'
    LLDP: Final[str] = '0.9.3-20240320'
    LINUX_IP_ADDRESSES: Final[str] = '0.0.4-20241210'
    SNMP_IP_ADDRESSES: Final[str] = '0.0.6-20241210'
    WINDOWS_IP_ADDRESSES: Final[str] = '0.0.3-20241210'


@unique
class TomlSections(EnumValue):
    EMBLEMS: Final[str] = 'EMBLEMS'
    FILTER_BY_CUSTOMER: Final[str] = 'FILTER_BY_CUSTOMER'
    FILTER_BY_FOLDER: Final[str] = 'FILTER_BY_FOLDER'
    FILTER_BY_HOST_LABEL: Final[str] = 'FILTER_BY_HOST_LABEL'
    FILTER_BY_HOST_TAG: Final[str] = 'FILTER_BY_HOST_TAG'
    FILTER_BY_SITE: Final[str] = 'FILTER_BY_SITE'
    L2_DROP_NEIGHBOURS: Final[str] = 'L2_DROP_NEIGHBOURS'
    L2_NEIGHBOUR_REPLACE_REGEX: Final[str] = 'L2_NEIGHBOUR_REPLACE_REGEX'
    L2_NEIGHBOUR_TO_HOST_MAP: Final[str] = 'L2_NEIGHBOUR_TO_HOST_MAP'
    L2_SEED_DEVICES: Final[str] = 'L2_SEED_DEVICES'
    L3V4_IGNORE_WILDCARD: Final[str] = 'L3V4_IGNORE_WILDCARD'
    L3_IGNORE_HOSTS: Final[str] = 'L3_IGNORE_HOSTS'
    L3_IGNORE_IP: Final[str] = 'L3_IGNORE_IP'
    L3_REPLACE_NETWORKS: Final[str] = 'L3_REPLACE_NETWORKS'
    L3_SUMMARIZE: Final[str] = 'L3_SUMMARIZE'
    MAP_SPEED_TO_THICKNESS: Final[str] = 'MAP_SPEED_TO_THICKNESS'
    PROTECTED_TOPOLOGIES: Final[str] = 'PROTECTED_TOPOLOGIES'
    SETTINGS: Final[str] = 'SETTINGS'
    STATIC_CONNECTIONS: Final[str] = 'STATIC_CONNECTIONS'


def cli_long_to_toml(cli_param: str) -> str:
    return cli_param.strip('-').replace('-', '_')


@unique
class TomlSettings(EnumValue):
    API_PORT: Final[str] = cli_long_to_toml(CliLong.API_PORT)
    BACKEND: Final[str] = cli_long_to_toml(CliLong.BACKEND)
    CHECK_CONFIG: Final[str] = cli_long_to_toml(CliLong.CHECK_CONFIG)
    CONFIG: Final[str] = cli_long_to_toml(CliLong.CONFIG)
    DEFAULT: Final[str] = cli_long_to_toml(CliLong.DEFAULT)
    DONT_COMPARE: Final[str] = cli_long_to_toml(CliLong.DONT_COMPARE)
    FILTER_CUSTOMERS: Final[str] = cli_long_to_toml(CliLong.FILTER_CUSTOMERS)
    FILTER_SITES: Final[str] = cli_long_to_toml(CliLong.FILTER_SITES)
    KEEP_MAX_TOPOLOGIES: Final[str] = cli_long_to_toml(CliLong.KEEP_MAX_TOPOLOGIES)
    L2_CASE: Final[str] = cli_long_to_toml(CliLong.L2_CASE)
    L2_DISPLAY_NEIGHBOURS: Final[str] = cli_long_to_toml(CliLong.L2_DISPLAY_NEIGHBOURS)
    L2_DISPLAY_PORTS: Final[str] = cli_long_to_toml(CliLong.L2_DISPLAY_PORTS)
    L2_IGNORE_MISMATCH: Final[str] = cli_long_to_toml(CliLong.L2_IGNORE_MISMATCH)
    L2_PREFIX: Final[str] = cli_long_to_toml(CliLong.L2_PREFIX)
    L2_REMOVE_DOMAIN: Final[str] = cli_long_to_toml(CliLong.L2_REMOVE_DOMAIN)
    L2_SKIP_EXTERNAL: Final[str] = cli_long_to_toml(CliLong.L2_SKIP_EXTERNAL)
    L3_DISPLAY_DEVICES: Final[str] = cli_long_to_toml(CliLong.L3_DISPLAY_DEVICES)
    L3_INCLUDE_HOSTS: Final[str] = cli_long_to_toml(CliLong.L3_INCLUDE_HOSTS)
    L3_INCLUDE_LOOPBACK: Final[str] = cli_long_to_toml(CliLong.L3_INCLUDE_LOOPBACK)
    L3_SKIP_CIDR_0: Final[str] = cli_long_to_toml(CliLong.L3_SKIP_CIDR_0)
    L3_SKIP_CIDR_32_128: Final[str] = cli_long_to_toml(CliLong.L3_SKIP_CIDR_32_128)
    L3_SKIP_IF: Final[str] = cli_long_to_toml(CliLong.L3_SKIP_IF)
    L3_SKIP_IP: Final[str] = cli_long_to_toml(CliLong.L3_SKIP_IP)
    L3_SKIP_PUBLIC: Final[str] = cli_long_to_toml(CliLong.L3_SKIP_PUBLIC)
    LAYERS: Final[str] = cli_long_to_toml(CliLong.LAYERS)
    LOG_FILE: Final[str] = cli_long_to_toml(CliLong.LOG_FILE)
    LOG_LEVEL: Final[str] = cli_long_to_toml(CliLong.LOG_LEVEL)
    LOG_TO_STDOUT: Final[str] = cli_long_to_toml(CliLong.LOG_TO_STDOUT)
    MIN_TOPOLOGY_AGE: Final[str] = cli_long_to_toml(CliLong.MIN_TOPOLOGY_AGE)
    OUTPUT_DIRECTORY: Final[str] = cli_long_to_toml(CliLong.OUTPUT_DIRECTORY)
    PRE_FETCH: Final[str] = cli_long_to_toml(CliLong.PRE_FETCH)
    QUIET: Final[str] = cli_long_to_toml(CliLong.QUIET)
    TIME_FORMAT: Final[str] = cli_long_to_toml(CliLong.TIME_FORMAT)
    UPDATE_CONFIG: Final[str] = cli_long_to_toml(CliLong.UPDATE_CONFIG)
